% \dot{x} = x(1 - x - a*y) - (d*y*x^2)/(k^2 + x^2)      + g1*W1 + J1;
% \dot{y} = r*y(1 - b*x - y) + (e*d*y*x^2)/(k^2 + x^2)  + g2*W2 + J2;
% Parameters: % a = .7, d = .075; k = .1; b = .9, r = 1.5, e = .01;
% g = diffusion; J = jump;
close all; clear all; clc; 

addpath(genpath('.\SeDuMi_1_3')); addpath(genpath('.\SOSTOOLS.204\'));

pvar x1 x2 t;
% par = [a b d e k r deg diff_sigma]
par = [.7 .9 .075 .01 .1 1.5 2 0]; 
den = par(5)^2+x1^2; g = par(8)*den*[1;1]; 

% jump = [\mu; \lambda; \rho];
jump = [10; 1; 0];

% vec. fields
f = [x1*(1-x1-par(1)*x2)*den-par(3)*x1^2*x2;
    par(6)*x2*(1-par(2)*x1-x2)*den + par(4)*par(3)*x1^2*x2; 
    .01*den];

%sets
X = [x1*(1-x1); x2*(1-x2); t*(5-t)];        % X
X_0 = -(x1-.71)^2 - (x2-.36)^2 + (.01)^2;   % X_0
X_u = [x2 - 0.1398*x1^3 - 9.5102*x1^2 - 1.0921*x1 + 0.0003; %\partial X 
    x1*(.272-x1); x2*(1-x2)];

mfpt = zeros(1,6);
for i = 1:6
    rho = jump(2)*i;
    prog = sosprogram([x1; x2; t]);

    % V(x) >=0; x \in X
    [prog,mu1] = sossosvar(prog,monomials([x1; x2; t],0:par(7)/2-1));
    [prog,mu2] = sossosvar(prog,monomials([x1; x2; t],0:par(7)/2-1));
    [prog,mu3] = sossosvar(prog,monomials([x1; x2; t],0:par(7)/2-1));
    [prog,V] = sossosvar(prog,monomials([x1; x2; t],0:par(7)/2));
    expr1 = V - mu1*X(1) - mu2*X(2) - mu3*X(3);
    prog = sosineq(prog,expr1);

     J = Njump_gen(V,par(7),1/jump(1)); % jumps

    % V(x) <= 0, x \in X_u
    [prog,mu1] = sossosvar(prog,monomials([x1; x2],0:par(7)/2-1));
    [prog,mu2] = sossosvar(prog,monomials([x1; x2],0:par(7)/2-1));
    [prog,mu3] = sossosvar(prog,monomials([x1; x2],0:par(7)/2-1));
    expr2 = subs(V,t,5) - mu1*X_u(1) - mu2*X_u(2) - mu3*X_u(3);
    prog = sosineq(prog,expr2);

    % V(x) <=0 gamma, x \in X_0
    [prog,gamma] = sospolyvar(prog,1);
    % prog = sosineq(prog,5-gamma)
    [prog,mu1] = sossosvar(prog,monomials([x1;x2;t],0:par(7)/2-1));
    expr3 = - V - mu1*X_0 + gamma;
    prog = sosineq(prog,expr3);

    % dB/dt + L*V <= -1
    [prog,mu1] = sossosvar(prog,monomials([x1; x2; t],0:par(7)/2-1));
    [prog,mu2] = sossosvar(prog,monomials([x1; x2; t],0:par(7)/2-1));
    [prog,mu3] = sossosvar(prog,monomials([x1; x2; t],0:par(7)/2-1));
    expr4 = -(diff(V,x1)*f(1) + diff(V,x2)*f(2) + diff(V,t)*f(3) ...
        + .5*g'*(diff(diff(V,x1),x1) + diff(diff(V,x2),x2))*g)...
        - (mu1*X(1) + mu2*X(2) + mu3*X(3)) - 1  - rho*J;
    prog = sosineq(prog,expr4);

    prog = sossetobj(prog,gamma);   % bounds
    prog = sossolve(prog);
    mfpt(i) = sosgetsol(prog,gamma)*10; 
end

figure
plot(1:1:6,mfpt,'ok'); axis([.5 6.5 0 8])
xlabel('\lambda'); ylabel('E[\tau]');